@@ -30,7 +30,7 @@ from thunder.config import Config
 BASEURL = "https://api.thundercompute.com:8443"
 # For debug mode
 if os.environ.get('API_DEBUG_MODE') == "1":
-    BASEURL = 'http://localhost:8080'
+    BASEURL = 'http://localhost:8443'
 
 PLATFORM = "unknown"
 try:
@@ -217,6 +217,9 @@ def configure_thunder_virtualization(ssh, token, metadata, instance_id, latest_b
                     update_binary = True
 
             if update_binary:
+                # Check if this is a K8s instance (config provided by ConfigMap)
+                is_k8s_instance = metadata.get('k8s', False)
+                
                 config = {
                     "instanceId": instance_id,
                     "deviceId": device_id,
@@ -237,8 +240,9 @@ def configure_thunder_virtualization(ssh, token, metadata, instance_id, latest_b
                     f'sudo install -d -m 755 {thunder_dir}',
                     'sudo install -d -m 755 /home/ubuntu/.thunder',
 
-                    # Back up existing preload file
-                    'sudo cp -a /etc/ld.so.preload /etc/ld.so.preload.bak || true',
+                    # Back up existing preload file, then remove the original
+                    'sudo cp -a /etc/ld.so.preload /etc/ld.so.preload.bak',     
+                    'sudo rm -f /etc/ld.so.preload',        
 
                     # Download to a temporary file first
                     f'curl -L https://storage.googleapis.com/client-binary/client_linux_x86_64 -o /tmp/libthunder.tmp',
@@ -257,26 +261,32 @@ def configure_thunder_virtualization(ssh, token, metadata, instance_id, latest_b
                     'sudo chown root:root /etc/ld.so.preload.new',
                     'sudo chmod 644 /etc/ld.so.preload.new',
                     'sudo mv /etc/ld.so.preload.new /etc/ld.so.preload',
+                ]
 
-                    # Write config files
-                    f'echo \'{config_json}\' > /tmp/config.tmp',
-                    f'sudo install -m 644 -o root -g root /tmp/config.tmp {remote_config_path}',
-                    'rm -f /tmp/config.tmp',
-
+                # Add token writing for all instances (K8s and non-K8s need tokens)
+                commands.extend([
                     f'echo \'{token}\' > /tmp/token.tmp',
                     f'sudo install -m 600 -o root -g root /tmp/token.tmp {remote_token_path}',
                     'rm -f /tmp/token.tmp',
-
-                    # Symlink config and token to .thunder
-                    'sudo rm -f /etc/thunder/config.json || true',
-                    f'sudo ln -sf {remote_config_path} /etc/thunder/config.json',
-                    'sudo chown root:root /etc/thunder/config.json',
-                    'sudo chmod 644 /etc/thunder/config.json',
                     'sudo rm -f /etc/thunder/token || true',
                     f'sudo ln -sf {remote_token_path} /etc/thunder/token',
                     'sudo chown root:root /etc/thunder/token',
                     'sudo chmod 644 /etc/thunder/token'
-                ]
+                ])
+
+                # Add config writing only for non-K8s instances (K8s uses ConfigMap)
+                if not is_k8s_instance:
+                    commands.extend([
+                        # Write config files
+                        f'echo \'{config_json}\' > /tmp/config.tmp',
+                        f'sudo install -m 644 -o root -g root /tmp/config.tmp {remote_config_path}',
+                        'rm -f /tmp/config.tmp',
+                        # Symlink config to .thunder
+                        'sudo rm -f /etc/thunder/config.json || true',
+                        f'sudo ln -sf {remote_config_path} /etc/thunder/config.json',
+                        'sudo chown root:root /etc/thunder/config.json',
+                        'sudo chmod 644 /etc/thunder/config.json',
+                    ])
 
                 command_string = ' && '.join(commands)
                 _, stdout, stderr = ssh.exec_command(command_string)
@@ -676,7 +686,7 @@ def write_ssh_config(lines):
         raise
 
 
-def add_instance_to_ssh_config(hostname, key_path, host_alias=None, port=None):
+def add_instance_to_ssh_config(hostname, key_path, host_alias=None, port=None, local_forward_ports=None):
     """Add instance to SSH config with proper validation and formatting."""
     if not hostname or not key_path:
         raise ValueError("Hostname and key_path are required")
@@ -700,8 +710,14 @@ def add_instance_to_ssh_config(hostname, key_path, host_alias=None, port=None):
         f"    StrictHostKeyChecking no\n",
     ]
 
-    if port:
-        new_entry.append(f"    LocalForward {port} localhost:{port}\n")
+    # Add SSH connection port if different from 22
+    if port and port != 22:
+        new_entry.append(f"    Port {port}\n")
+    
+    # Add local port forwarding for specified ports
+    if local_forward_ports:
+        for forward_port in local_forward_ports:
+            new_entry.append(f"    LocalForward {forward_port} localhost:{forward_port}\n")
 
     config_lines.extend(new_entry)
     write_ssh_config(config_lines)
@@ -753,8 +769,8 @@ def get_ssh_config_entry(instance_name):
         return False, None
 
 
-def update_ssh_config_ip(instance_name, new_ip_address, keyfile=None):
-    """Update instance IP and optionally key file in SSH config atomically."""
+def update_ssh_config_ip(instance_name, new_ip_address, keyfile=None, port=None):
+    """Update instance IP, port, and optionally key file in SSH config atomically."""
     if not instance_name or not new_ip_address:
         return
 
@@ -762,6 +778,7 @@ def update_ssh_config_ip(instance_name, new_ip_address, keyfile=None):
     new_lines = []
     in_target_host = False
     updated_ip = False
+    updated_port = False if port else True  # If no port, consider it updated
     has_strict_checking = False
     updated_key = False if keyfile else True  # If no keyfile, consider it updated
 
@@ -774,9 +791,12 @@ def update_ssh_config_ip(instance_name, new_ip_address, keyfile=None):
                     new_lines.append("    StrictHostKeyChecking no\n")
                 if keyfile and not updated_key:
                     new_lines.append(f"    IdentityFile \"{keyfile}\"\n")
+                if port and not updated_port:
+                    new_lines.append(f"    Port {port}\n")
             in_target_host = (stripped == f"Host {instance_name}")
             has_strict_checking = False
             updated_key = False if keyfile else True
+            updated_port = False if port else True
             new_lines.append(line)
             continue
 
@@ -785,6 +805,10 @@ def update_ssh_config_ip(instance_name, new_ip_address, keyfile=None):
                 new_lines.append(f"    HostName {new_ip_address}\n")
                 updated_ip = True
                 continue
+            if port and stripped.startswith("Port") and not updated_port:
+                new_lines.append(f"    Port {port}\n")
+                updated_port = True
+                continue
             if stripped.startswith("StrictHostKeyChecking"):
                 has_strict_checking = True
             if keyfile and stripped.startswith("IdentityFile"):
@@ -800,6 +824,8 @@ def update_ssh_config_ip(instance_name, new_ip_address, keyfile=None):
             new_lines.append("    StrictHostKeyChecking no\n")
         if keyfile and not updated_key:
             new_lines.append(f"    IdentityFile \"{keyfile}\"\n")
+        if port and not updated_port:
+            new_lines.append(f"    Port {port}\n")
 
     if updated_ip:
         write_ssh_config(new_lines)
